SystemOrganization addCategory: #'AL-Http-Core'!
SystemOrganization addCategory: #'AL-Http-Client'!
SystemOrganization addCategory: #'AL-Http-Server'!
SystemOrganization addCategory: #'AL-Http-Protocol'!

Object subclass: #ALClient
	instanceVariableNames: 'host port in out'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Client'!

!ALClient class methodsFor: 'as yet unclassified' stamp: 'cwp 9/6/2011 16:38'!
host: aString
	^ self host: aString port: 80 ! !

!ALClient class methodsFor: 'as yet unclassified' stamp: 'cwp 9/4/2011 00:09'!
host: aString port: anInteger
	^ self host: aString port: anInteger in: nil out: nil! !

!ALClient class methodsFor: 'as yet unclassified' stamp: 'cwp 9/4/2011 00:08'!
host: aString port: anInteger in: aReadStream out: aWriteStream
	^ self basicNew
		initializeWithHost: aString
		port: anInteger
		in: aReadStream
		out: aWriteStream! !

!ALClient methodsFor: 'private' stamp: 'cwp 9/6/2011 17:11'!
addHostToRequest: aRequest
	| ws |
	ws := String new writing.
	ws write: host.
	port = 80 ifFalse:
		[ws put: $:.
		ws print: port].
	aRequest host: ws conclusion! !

!ALClient methodsFor: 'connection' stamp: 'cwp 9/6/2011 16:13'!
connect
	self isConnected ifFalse:
		[| sock |
		Socket initializeNetwork.
		sock := Socket tcpCreateIfFail: [self error: 'Unable to create a socket'].
		sock connectToHostNamed: host port: port.
		in := sock reading.
		out := sock writing]! !

!ALClient methodsFor: 'connection' stamp: 'cwp 9/6/2011 16:32'!
disconnect
	in ifNotNil:
		[in close.
		in := nil].
	out ifNotNil:
		[out close.
		out := nil]! !

!ALClient methodsFor: 'public' stamp: 'cwp 9/6/2011 15:50'!
get: aPath
	| request response |
	request := ALRequest get: aPath.
	response := self sendRequest: request.
	^ response content! !

!ALClient methodsFor: 'accessing' stamp: 'cwp 9/4/2011 00:10'!
host
	^ host! !

!ALClient methodsFor: 'initialize-release' stamp: 'cwp 9/4/2011 00:10'!
initializeWithHost: aString port: anInteger in: aReadStream out: aWriteStream
	self initialize.
	host := aString.
	port := anInteger.
	in := aReadStream.
	out := aWriteStream! !

!ALClient methodsFor: 'connection' stamp: 'cwp 9/6/2011 17:03'!
isConnected
	in ifNil: [^ false].
	out ifNil: [^ false].
	^ true! !

!ALClient methodsFor: 'accessing' stamp: 'cwp 9/4/2011 00:10'!
port
	^ port! !

!ALClient methodsFor: 'public' stamp: 'cwp 9/6/2011 17:06'!
sendRequest: aRequest
	| printer parser response |
	self connect.
	self addHostToRequest: aRequest.
	printer := ALHttpPrinter on: out.
	parser := ALHttpParser on: in.
	printer printRequest: aRequest.
	response := parser parseResponse.
	^ response! !

Object subclass: #ALConnection
	instanceVariableNames: 'in out application'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Server'!

!ALConnection class methodsFor: 'as yet unclassified' stamp: 'cwp 4/24/2011 23:17'!
in: aReadStream out: aWriteStream application: anApplication
	^ self basicNew 
		initializeWithIn: aReadStream 
		out: aWriteStream 
		application: anApplication ! !

!ALConnection class methodsFor: 'instance creation' stamp: 'cwp 4/24/2011 23:17'!
socket: aSocket application: anApplication
	^ self in: aSocket reading out: aSocket writing application: anApplication! !

!ALConnection methodsFor: 'as yet unclassified' stamp: 'cwp 4/13/2011 21:54'!
close
	in close.
	out close.
	! !

!ALConnection methodsFor: 'running' stamp: 'cwp 8/20/2011 23:59'!
handleRequest
	| parser printer response |
	parser := ALHttpParser on: in.
	printer := ALHttpPrinter on: out.
	response := self handleRequest: parser parseRequest.
	printer printResponse: response.
	response writeEntityInto: out.
	response isConnectionClose ifTrue: [ self close ]! !

!ALConnection methodsFor: 'running' stamp: 'cwp 8/21/2011 00:01'!
handleRequest: aRequest
	^ [application handleRequest: aRequest]
		on: Error
		do: [:error | ALResponse status: 500 reason: error messageText]! !

!ALConnection methodsFor: 'running' stamp: 'cwp 4/13/2011 21:57'!
handleRequests
	[
		[[self handleRequest] repeat]
			on: Incomplete
			do: [:ex | self close]	
	] ifCurtailed: [self close]! !

!ALConnection methodsFor: 'initialize-release' stamp: 'cwp 4/24/2011 23:16'!
initializeWithIn: aReadStream out: aWriteStream application: anApplication
	self initialize.
	in := aReadStream.
	out := aWriteStream.
	application := anApplication ! !

Object subclass: #ALHttpParser
	instanceVariableNames: 'in ch message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Protocol'!

!ALHttpParser class methodsFor: 'as yet unclassified' stamp: 'cwp 4/5/2011 19:51'!
on: aReadStream
	^ self basicNew initializeWithStream: aReadStream! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assert: aByte
	ch = aByte ifFalse: [ALMalformedRequest signal].
	ch := nil.! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assertCrLf
	"Character cr asciiValue = 13"
	"Character lf acsiiValue = 10"
	
	self assert: 13.
	self next: 10.
! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assertSpace
	"Character space asciiValue = 32"
	self assert: 32.
! !

!ALHttpParser methodsFor: 'as yet unclassified' stamp: 'cwp 4/5/2011 19:52'!
initializeWithStream: aReadStream
	in := aReadStream
	! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 9/4/2011 23:42'!
next: expected
	| actual |
	actual := in get.
	actual = expected ifFalse: [ALMalformedRequest signal]! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/6/2011 10:34'!
parseFieldValue
	"Character tab asciiValue = 9"
	"Character space asciiValue = 32"
	"Character cr asciivalue = 13"
	"Character lf asciiValue = 10"
	"For now, we ignore the possibility of line folding"

	| out byte |
	[#(32 9) includes: (byte := in get)] whileTrue.
	out := ByteArray new writing.
	out put: byte.
	[(byte := in get) = 13] whileFalse: [out put: byte].
	self next: 10.
	^ out conclusion asString! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/4/2011 23:27'!
parseHeaders
	"$: asciiValue -> 58"
	
	| name value |
	[name := self parseToken.
	name isEmpty ifTrue: [^ self assertCrLf].
	self assert: 58.
	value := self parseFieldValue.
	message header: name put: value]
		repeat
	
! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 19:58'!
parseMethod
	| method |
	method := self parseToken.
	self assertSpace.
	^ method! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:00'!
parsePath
	"$/ asciiValue = 47"
	"Character space asciiValue = 32"

	| out path |
	path := ALPath root.
	ch := in get. 
	ch = 47 ifFalse: [self error].

	out := String new writing.
	[(ch := in get) = 32] whileFalse:
		[ch = 47
			ifFalse: [out put: (Character value: ch)]
			ifTrue: 
				[path := path / out conclusion.
				out := String new writing]].
	path := path / out conclusion.
	self assertSpace.
	^ path
			! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/4/2011 23:48'!
parseReason
	^ (in ending: 13) rest asString! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/4/2011 23:50'!
parseRequest
	message := self parseRequestLine.
	self parseHeaders.
	^ message! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/5/2011 16:09'!
parseRequestLine
	| version path method entity |
	method := self parseMethod.
	path := self parsePath.
	version := self parseVersion.
	self assertCrLf.
	entity := ALEntity reading: in.
	^ ALRequest method: method path: path version: version body: entity.
	! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/4/2011 23:51'!
parseResponse
	message := self parseStatusLine.
	self parseHeaders.
	^ message! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/4/2011 23:45'!
parseStatus
	| digits |
	digits := self readAll: #[48 49 50 51 52 53 54 55 56 57].
	^ Integer readFrom: digits readStream! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/5/2011 16:09'!
parseStatusLine
	| version reason status body |
	version := self parseVersion.
	self assert: 32.
	status := self parseStatus.
	self assert: 32.
	reason := self parseReason.
	self next: 10.
	body := ALEntity reading: in.
	^ ALResponse status: status reason: reason version: version body: body! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 19:57'!
parseToken
	"These are the ascii values of characters that may be part of a token"

	|  legal out |
	legal :=  #(33 35 36 37 38 39 42 43 45 46 48 49 50 51 52 53 54 55 56 57 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 124 126).
	
	out := ByteArray new writing.
	ch ifNotNil: [out put: ch].
	[legal includes: (ch := in get)] whileTrue: [out put: ch].
	^ out conclusion asString! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/4/2011 23:41'!
parseVersion
	| version |
	self next: $H asciiValue.
	self next: $T asciiValue.
	self next: $T asciiValue.
	self next: $P asciiValue.
	self next: $/ asciiValue.
	version := self readAll: #[48 49 50 51 52 53 54 55 56 57 46].
	^ version! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:12'!
readAll: aByteArray
	| out |
	out := ByteArray new writing.
	[aByteArray includes: (ch := in get)] whileTrue: [out put: ch].
	^ out conclusion asString! !

Object subclass: #ALHttpPrinter
	instanceVariableNames: 'out message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Protocol'!

!ALHttpPrinter class methodsFor: 'as yet unclassified' stamp: 'cwp 4/23/2011 15:54'!
on: aWriteStream
	^ self basicNew initializeWithStream: aWriteStream ! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 23:20'!
crlf

	out
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'initialize-release' stamp: 'cwp 4/23/2011 15:55'!
initializeWithStream: aWriteStream
	out := aWriteStream  ! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 23:21'!
printHeader: name value: value
	out
		write: name;
		put: 58;
		put: 32;
		write: value;
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 23:21'!
printHeaders
	message headersDo: [ :name :value | self printHeader: name value: value ].
	out
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/6/2011 16:39'!
printRequest
	self printRequestLine.
	self printHeaders.
	out flush! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 01:30'!
printRequest: aRequest 
	message := aRequest.
	self printRequest.
	message := nil! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 23:21'!
printRequestLine
	"Character space asciiValue -> 32"

	out
		write: message method;
		space;
		print: message path;
		space.
	self printVersion.
	out
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/6/2011 16:26'!
printResponse
	self printStatusLine.
	self printHeaders.
	out flush! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 01:30'!
printResponse: aResponse
	message := aResponse.
	self printResponse.
	message := nil! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 01:35'!
printStatusLine
	self printVersion.
	out
		space;
		print: message status;
		space;
		write: message reason;
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 01:35'!
printVersion
	out
		write: 'HTTP/';
		write: (String streamContents: [ :s | message version printFractionAsDecimalOn: s ])! !

Object subclass: #ALMimeMessage
	instanceVariableNames: 'headers body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

ALMimeMessage subclass: #ALEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

ALEntity subclass: #ALEmptyEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALEmptyEntity methodsFor: 'as yet unclassified' stamp: 'cwp 9/5/2011 16:48'!
readWith: aBlock
	aBlock value: #[] reading! !

!ALEmptyEntity methodsFor: 'as yet unclassified' stamp: 'cwp 9/5/2011 16:47'!
writeInto: aWriteStream
	! !

!ALEntity class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 16:18'!
body: anObject
	^ self basicNew initializeWithBody: anObject! !

!ALEntity class methodsFor: 'writing' stamp: 'cwp 9/5/2011 16:34'!
callback: aBlock
	^ ALWriteEntity body: aBlock! !

!ALEntity class methodsFor: 'writing' stamp: 'cwp 9/5/2011 16:48'!
empty
	^ ALEmptyEntity body: nil! !

!ALEntity class methodsFor: 'writing' stamp: 'cwp 4/23/2011 19:47'!
html: aString
	^ (self callback: [:out | (out encoding: #ascii) write: aString])
		header: 'Content-Type' put: 'text/html';
		header: 'Content-Length' put: aString length printString;
		yourself! !

!ALEntity class methodsFor: 'reading' stamp: 'cwp 9/5/2011 16:34'!
reading: aReadable
	^ ALReadEntity body: aReadable! !

!ALEntity methodsFor: 'headers - getting' stamp: 'cwp 9/6/2011 15:57'!
contentEncoding
	^ self header: 'Content-Encoding'! !

!ALEntity methodsFor: 'headers - getting' stamp: 'cwp 9/5/2011 15:07'!
contentLength
	| string |
	string := (self header: 'Content-Length') ifNil: [^ nil].
	^ Integer readFrom: string readStream! !

!ALEntity methodsFor: 'headers - setting' stamp: 'cwp 8/9/2011 23:57'!
contentLength: anInteger 
	self header: 'Content-Length' put: anInteger printString! !

!ALEntity methodsFor: 'headers - getting' stamp: 'cwp 9/5/2011 15:12'!
contentType
	^ self header: 'Content-Type'! !

!ALEntity methodsFor: 'headers - setting' stamp: 'cwp 5/2/2011 18:31'!
contentType: aString 
	self header: 'Content-Type' put: aString! !

!ALEntity methodsFor: 'io' stamp: 'cwp 9/5/2011 16:36'!
readWith: aBlock
	self subclassResponsibility ! !

!ALEntity methodsFor: 'io' stamp: 'cwp 9/5/2011 16:36'!
writeInto: aWriteStream
	self subclassResponsibility ! !

ALEntity subclass: #ALReadEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALReadEntity methodsFor: 'as yet unclassified' stamp: 'cwp 9/5/2011 16:51'!
readWith: aBlock
	| stream |
	stream := self reading.
	^ [aBlock value: stream]
		ensure: [stream rest]
	! !

!ALReadEntity methodsFor: 'as yet unclassified' stamp: 'cwp 9/6/2011 16:03'!
reading
	| limit stream encoding |
	limit := self contentLength.
	encoding := self contentEncoding.
	
	
	stream := body reading.
	limit ifNotNil: [ stream := stream limiting: limit ].
	encoding 
		ifNotNil: [ stream := stream encoding: encoding ]
		ifNil: 
			[| type |
			type := self contentType.
			type ifNotNil: 
				[(type beginsWith: 'text/') ifTrue: 
					[stream := stream encoding: #utf8]]].
		
	^ stream! !

!ALReadEntity methodsFor: 'as yet unclassified' stamp: 'cwp 9/5/2011 16:16'!
writeInto: aWriteStream
	aWriteStream write: self reading! !

ALEntity subclass: #ALWriteEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALWriteEntity methodsFor: 'as yet unclassified' stamp: 'cwp 9/5/2011 16:51'!
readWith: aBlock
	| stream limit |
	limit := self contentLength.
	stream := [ :out | self writeInto: out ] reading.
	stream contentsSpecies: ByteArray.
	limit ifNotNil: [stream := stream limiting: limit].
	^ [aBlock value: stream]
		ensure: [limit ifNotNil: [stream rest]].! !

!ALWriteEntity methodsFor: 'as yet unclassified' stamp: 'cwp 9/5/2011 16:36'!
writeInto: aWriteStream
	| limit stream |
	limit := self contentLength.
	stream := limit ifNil: [aWriteStream] ifNotNil: [aWriteStream limiting: limit].
	body value: stream.
	! !

ALMimeMessage subclass: #ALHttpMessage
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALHttpMessage class methodsFor: 'constants' stamp: 'cwp 9/5/2011 14:20'!
defaultVersion
	^ 1.1s1! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 5/4/2011 23:08'!
beConnectionKeepAlive
	self header: 'Connection' put: 'keep-alive'! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 5/4/2011 22:02'!
beTransferEncodingChunked
	self header: 'Transfer-Encoding' put: 'chunked'! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 9/6/2011 15:50'!
content
	^ self readEntityWith: [:in | in rest]! !

!ALHttpMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/5/2011 14:46'!
header: aString put: anObject
	(self class entityHeaders includes: aString)
		ifTrue: [body header: aString put: anObject]
		ifFalse: [headers at: aString put: anObject]! !

!ALHttpMessage methodsFor: 'headers - accessing' stamp: 'cwp 5/4/2011 22:43'!
headersDo: aBlock
	super headersDo: aBlock.
	body headersDo: aBlock.! !

!ALHttpMessage methodsFor: 'initialize-release' stamp: 'cwp 9/5/2011 14:11'!
initializeWithVersion: aScaledDecimal body: anEntity
	self initializeWithBody: anEntity.
	version := aScaledDecimal ! !

!ALHttpMessage methodsFor: 'headers - testing' stamp: 'cwp 5/4/2011 23:07'!
isConnectionClose
	^ (self header: 'Connection') == 'close'! !

!ALHttpMessage methodsFor: 'headers - testing' stamp: 'cwp 4/24/2011 21:22'!
isConnectionKeepAlive
	^ (self header: 'Connection') = 'keep-alive'! !

!ALHttpMessage methodsFor: 'headers - testing' stamp: 'cwp 5/2/2011 18:37'!
isTransferEncodingChunked
	^ (self header: 'Transfer-Encoding') = 'chunked'! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 9/6/2011 15:51'!
readEntityWith: aBlock
	^ body readWith: aBlock! !

!ALHttpMessage methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:16'!
version
	^ version! !

!ALHttpMessage methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:16'!
version: anObject
	version := anObject! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 9/5/2011 14:48'!
writeEntityInto: aWriteStream
	self isTransferEncodingChunked
		ifFalse: [body writeInto: aWriteStream]
		ifTrue: 
			[| stream |
			stream := (aWriteStream closing: []) chunking.
			body writeInto: stream.
			stream close]! !

ALHttpMessage subclass: #ALRequest
	instanceVariableNames: 'method path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALRequest class methodsFor: 'convenience' stamp: 'cwp 9/4/2011 01:58'!
get: aPath 
	^ (self method: 'GET' path: aPath)
		userAgent: self userAgent;
		acceptAnything;
		yourself! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:21'!
method: aString path: aPath
	^ self
		method: aString
		path: aPath
		version: self defaultVersion
		body: ALEntity empty! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:20'!
method: aString path: aPath version: aScaledDecimal 
	^ self 
		method: aString 
		path: aPath 
		version: aScaledDecimal 
		body: ALEntity empty! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:19'!
method: aString path: aPath version: aScaledDecimal body: anEntity
	^ self basicNew
		initializeWithMethod: aString 
		path: aPath 
		version: aScaledDecimal 
		body: anEntity ! !

!ALRequest class methodsFor: 'accessing' stamp: 'cwp 9/4/2011 02:01'!
userAgent
	^ 'Altitude/0.1'! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 9/4/2011 02:02'!
accept: aString
	self header: 'Accept' put: aString! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 9/4/2011 02:00'!
acceptAnything
	self accept: '*/*'! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 9/6/2011 17:09'!
host: aString
	self header: 'Host' put: aString! !

!ALRequest methodsFor: 'initialize-release' stamp: 'cwp 4/24/2011 21:05'!
initializeWithMethod: aString path: aPath 
	self initializeWithBody: ALEntity empty.
	method := aString.
	path := aPath.
	! !

!ALRequest methodsFor: 'initialize-release' stamp: 'cwp 9/5/2011 14:13'!
initializeWithMethod: aString path: aPath version: aScaledDecimal body: anEntity
	self initializeWithVersion: aScaledDecimal body: anEntity.
	method := aString.
	path := aPath.
	! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 3/21/2011 22:25'!
isMethodGet
	^ method = 'GET'! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:01'!
method
	^ method! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 3/20/2011 22:58'!
path
	^ path! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 9/4/2011 02:02'!
userAgent: aString
	self header: 'User-Agent' put: aString! !

ALHttpMessage subclass: #ALResponse
	instanceVariableNames: 'status reason'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 4/13/2011 23:01'!
notFound: aPath
	^ self status: 404 reason: 'Not found ', aPath printString! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 8/9/2011 16:00'!
seeOther: aPath
	^ (self status: 303 reason: 'See Other')
		header: 'Location' put: aPath printString;
		yourself! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:17'!
status: anInteger reason: aString
	^ self 	
		status: anInteger 
		reason: aString 
		version: self defaultVersion 
		body: ALEntity empty! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:18'!
status: anInteger reason: aString body: anEntity
	^ self 
		status: anInteger 
		reason: aString 
		version: self defaultVersion  
		body: anEntity! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 4/23/2011 19:43'!
status: status reason: aString details: aString2
	| html |
	html := String new writing
		write: '<!!DOCTYPE html><html><head><title>';
		print: status;
		space;
		write: aString;
		write: '</title></head><body><h1>';
		write: aString;
		write: '</h1><p>';
		write: aString2;
		write: '</p></body></html>'.
	^ self status: 404 reason: 'Not Found' body: (ALEntity html: html)! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:18'!
status: anInteger reason: aString version: aScaledDecimal
	^ self 
		status: anInteger 
		reason: aString 
		version: aScaledDecimal 
		body: ALEntity empty! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:14'!
status: anInteger reason: aString version: aScaledDecimal body: anEntity
	^ self basicNew
		initializeWithStatus: anInteger 
		reason: aString
		version: aScaledDecimal
		body: anEntity! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 5/2/2011 18:27'!
type: aString callback: aBlock
	| entity |
	entity := ALEntity callback: aBlock.
	entity contentType: aString.
	^ (self status: 200 reason: 'OK' body: entity)
		beTransferEncodingChunked;
		yourself! !

!ALResponse methodsFor: 'initialize-release' stamp: 'cwp 9/5/2011 14:29'!
initializeWithStatus: anInteger reason: aString version: aScaledDecimal body: anEntity
	self initializeWithVersion: aScaledDecimal body: anEntity.
	status := anInteger.
	reason := aString.
	! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 16:23'!
reason
	^ reason! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 16:23'!
status
	^ status! !

!ALMimeMessage class methodsFor: 'as yet unclassified' stamp: 'cwp 9/5/2011 14:45'!
entityHeaders
	^ #(
		'Content-Encoding' 
		'Content-Language'
		'Content-Length'
		'Content-Location'
		'Content-MD5'
		'Content-Range'
		'Content-Type'
		'Expires'
		'Last-Modified'
	)! !

!ALMimeMessage methodsFor: 'accessing' stamp: 'cwp 5/2/2011 18:33'!
body: anObject
	 body := anObject! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 4/23/2011 19:19'!
header: aString 
	^ headers at: aString ifAbsent: [nil]! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 4/23/2011 19:17'!
header: aString put: anObject
	headers at: aString put: anObject! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 4/23/2011 19:17'!
headersDo: aBlock
	headers keysAndValuesDo: aBlock  ! !

!ALMimeMessage methodsFor: 'initialize-release' stamp: 'cwp 4/24/2011 21:04'!
initializeWithBody: anObject
	self initialize.
	headers := Dictionary new.
	body := anObject.! !

Object subclass: #ALServer
	instanceVariableNames: 'port process socket state application'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Server'!

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:13'!
initialize
	Smalltalk addToStartUpList: self after: AutoStart.
	Smalltalk addToShutDownList: self! !

!ALServer class methodsFor: 'instance creation' stamp: 'cwp 4/23/2011 17:55'!
on: anInteger application: anApplication
	^ self basicNew initializeWithPort: anInteger application: anApplication! !

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:57'!
shutDown: aBoolean
	self allInstancesDo: [:ea | ea shutDown]! !

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:57'!
startUp: aBoolean
	self allInstancesDo: [:ea | ea startUp]! !

!ALServer methodsFor: 'initialize-release' stamp: 'cwp 5/4/2011 20:26'!
initializeWithPort: anInteger application: anApplication
	port := anInteger.
	application := anApplication.
	state := #stopped
	! !

!ALServer methodsFor: 'testing' stamp: 'cwp 4/13/2011 17:01'!
isRunning
	^ state == #running! !

!ALServer methodsFor: 'private' stamp: 'cwp 5/4/2011 20:23'!
loop
	[self serveConnection] repeat! !

!ALServer methodsFor: 'printing' stamp: 'cwp 4/13/2011 17:16'!
printOn: aStream
	aStream nextPutAll: (self streamingPrintOn: String new writing)! !

!ALServer methodsFor: 'private' stamp: 'cwp 5/4/2011 20:27'!
run

	socket := Socket tcpCreateIfFail: [self error: 'Unable to create a socket'].
	socket
		setOption: 'TCP_NODELAY' value: 1;
		setOption: 'SO_SNDBUF' value: 1024;
		setOption: 'SO_RCVBUF' value: 1024;
		listenOn: port backlogSize: 10.
	socket isValid ifFalse: [self error: 'Unable to listen on port ', port printString].
	
	[self loop] ifCurtailed:   
		[socket destroy.
		socket := nil.
		process := nil].

! !

!ALServer methodsFor: 'private' stamp: 'cwp 5/4/2011 21:42'!
serveConnection
	| connection |
	connection := socket waitForAcceptFor: self timeout.
	connection ifNil: [^ self].
	[
		[(ALConnection socket: connection application: application) handleRequests] 
			forkAt: Processor highIOPriority 
			named: 'ALConnection to ', connection remoteAddress printString.
			
	] ifCurtailed: [connection destroy]! !

!ALServer methodsFor: 'running' stamp: 'cwp 8/20/2011 23:49'!
shutDown
	state == #running ifTrue:
		[process ifNotNil: [process terminate].
		state := #suspended]! !

!ALServer methodsFor: 'running' stamp: 'cwp 5/4/2011 20:35'!
start
	self isRunning ifTrue: [self stop].	
	state := #running.
	process := [self run] 
		forkAt: Processor lowIOPriority 
		named: self printString.
	! !

!ALServer methodsFor: 'running' stamp: 'cwp 5/4/2011 20:30'!
startUp
	state == #suspended ifTrue: [self start]! !

!ALServer methodsFor: 'running' stamp: 'cwp 8/6/2011 16:12'!
stop
	process ifNotNil: [process terminate].
	state := #stopped! !

!ALServer methodsFor: 'printing' stamp: 'cwp 4/13/2011 17:18'!
streamingPrintOn: aWriteStream
	^ aWriteStream
		write: self class name;
		put: $[;
		print: port;
		put: $:;
		write: state;
		put: $];
		conclusion! !

!ALServer methodsFor: 'accessing' stamp: 'cwp 4/13/2011 17:07'!
timeout
	^ 5! !

Error subclass: #ALMalformedRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Protocol'!
ALServer initialize!
